using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TradingBlotter.Resources;
using System.IO;
using System.Runtime.InteropServices;
using QuartzTypeLib;



namespace TradingBlotter.WinGui.Forms
{
    public partial class VideoForm : Form
    {
        [DllImport("winmm.dll")]
        public static extern int waveOutGetVolume(IntPtr hwo, out uint dwVolume);

        [DllImport("winmm.dll")]
        public static extern int waveOutSetVolume(IntPtr hwo, uint dwVolume);

        private const int WM_APP = 0x8000;
        private const int WM_GRAPHNOTIFY = WM_APP + 1;
        private const int EC_COMPLETE = 0x00000001;
        private const int WS_CHILD = 0x40000000;
        private const int WS_CLIPCHILDREN = 0x2000000;
        //const int WM_GRAPHNOTIFY = 0x00008001;
        private const int WS_CLIPSIBLINGS = 0x04000000;
        private const int WM_MOVE = 0x00000003;


        private FilgraphManager m_objFilterGraph = null;
        private IBasicAudio m_objBasicAudio = null;
        private IVideoWindow m_objVideoWindow = null;
        private IMediaEvent m_objMediaEvent = null;
        private IMediaEventEx m_objMediaEventEx = null;
        private IMediaPosition m_objMediaPosition = null;
        private IMediaControl m_objMediaControl = null;

        enum MediaStatus { None, Stopped, Paused, Running };

        private MediaStatus m_CurrentStatus = MediaStatus.None;

        public VideoForm()
        {
            InitializeComponent();
            // By the default set the volume to 0
            uint CurrVol = 0;
            // At this point, CurrVol gets assigned the volume
            waveOutGetVolume(IntPtr.Zero, out CurrVol);
            // Calculate the volume
            ushort CalcVol = (ushort)(CurrVol & 0x0000ffff);
            // Get the volume on a scale of 1 to 10 (to fit the trackbar)
            trackWave.Value = CalcVol / (ushort.MaxValue / 10);

            cb1.SelectedIndex = 0;
            cb1.SelectedText = "Logo";

        }

        private void trackWave_Scroll(object sender, EventArgs e)
        {
            // Calculate the volume that's being set
            int NewVolume = ((ushort.MaxValue / 10) * trackWave.Value);
            // Set the same volume for both the left and the right channels
            uint NewVolumeAllChannels = (((uint)NewVolume & 0x0000ffff) | ((uint)NewVolume << 16));
            // Set the volume
            waveOutSetVolume(IntPtr.Zero, NewVolumeAllChannels);
        }

        private void btnFile_Click(object sender, EventArgs e)
        {
            LoadFile();

            UpdateStatusBar();
            UpdateToolBar();    


        }

        private void CleanUp()
        {
            if (m_objMediaControl != null)
                m_objMediaControl.Stop();

            m_CurrentStatus = MediaStatus.Stopped;

            if (m_objMediaEventEx != null)
                m_objMediaEventEx.SetNotifyWindow(0, 0, 0);

            if (m_objVideoWindow != null)
            {
                m_objVideoWindow.Visible = 0;
                m_objVideoWindow.Owner = 0;
            }

            if (m_objMediaControl != null) m_objMediaControl = null;
            if (m_objMediaPosition != null) m_objMediaPosition = null;
            if (m_objMediaEventEx != null) m_objMediaEventEx = null;
            if (m_objMediaEvent != null) m_objMediaEvent = null;
            if (m_objVideoWindow != null) m_objVideoWindow = null;
            if (m_objBasicAudio != null) m_objBasicAudio = null;
            if (m_objFilterGraph != null) m_objFilterGraph = null;
        }

        private void VideoForm_SizeChanged(object sender, EventArgs e)
        {
            if (m_objVideoWindow != null)
            {
                m_objVideoWindow.SetWindowPosition(panel1.ClientRectangle.Left,
                    panel1.ClientRectangle.Top,
                    panel1.ClientRectangle.Width,
                    panel1.ClientRectangle.Height);
            }
        }


        private void btnPlay_Click(object sender, EventArgs e)
        {
     
            m_objMediaControl.Run();
            m_CurrentStatus = MediaStatus.Running;
            
            UpdateStatusBar();
            UpdateToolBar();    
        }

        private void btnPause_Click(object sender, EventArgs e)
        {

               m_objMediaControl.Pause();
                    m_CurrentStatus = MediaStatus.Paused;


            UpdateStatusBar();
            UpdateToolBar();    
        }

        private void btnStop_Click(object sender, EventArgs e)
        {

             m_objMediaControl.Stop();
                    m_objMediaPosition.CurrentPosition = 0;
                    m_CurrentStatus = MediaStatus.Stopped;


            UpdateStatusBar();
            UpdateToolBar();    
        }

        private void LoadFile()
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            openFileDialog.Filter = "Media Files|*.mpg;*.mpeg;*.avi;*.wmv;*.wma;*.mov;*.wav;*.mp2;*.mp3|All Files|*.*";

            if (DialogResult.OK == openFileDialog.ShowDialog())
            {
                CleanUp();

                m_objFilterGraph = new FilgraphManager();
 
                m_objFilterGraph.RenderFile(openFileDialog.FileName);

                m_objBasicAudio = m_objFilterGraph as IBasicAudio;

                try
                {
                    m_objVideoWindow = m_objFilterGraph as IVideoWindow;
                    m_objVideoWindow.Owner = (int)panel1.Handle;
                    ////m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPCHILDREN;
                    //m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
                    //m_objVideoWindow.SetWindowPosition(panel1.ClientRectangle.Left,
                    //    panel1.ClientRectangle.Top,
                    //    panel1.ClientRectangle.Width,
                    //    panel1.ClientRectangle.Height);

                }
                catch (Exception)
                {
                    m_objVideoWindow = null;
                }

                m_objMediaEvent = m_objFilterGraph as IMediaEvent;

                m_objMediaEventEx = m_objFilterGraph as IMediaEventEx;
                m_objMediaEventEx.SetNotifyWindow((int)this.Handle, WM_GRAPHNOTIFY, 0);

                m_objMediaPosition = m_objFilterGraph as IMediaPosition;

                m_objMediaControl = m_objFilterGraph as IMediaControl;

                //this.Text = "DirectShow - [" + openFileDialog.FileName + "]";

                m_objMediaControl.Run();
                m_CurrentStatus = MediaStatus.Running;

                UpdateStatusBar();
                UpdateToolBar();


            }

        }

        public void SetLogo()
        {
            CleanUp();
            m_objFilterGraph = new FilgraphManager();
            m_objFilterGraph.RenderFile(Path.Combine(Application.StartupPath, @"Images\ftlogo.jpg"));
            m_objBasicAudio = m_objFilterGraph as IBasicAudio;

            try
            {
                m_objVideoWindow = m_objFilterGraph as IVideoWindow;
                m_objVideoWindow.Owner = (int)panel1.Handle;
                //m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPCHILDREN;
                m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
                m_objVideoWindow.SetWindowPosition(panel1.ClientRectangle.Left,
                    panel1.ClientRectangle.Top,
                    panel1.ClientRectangle.Width,
                    panel1.ClientRectangle.Height);                 
            }
            catch (Exception)
            {
                m_objVideoWindow = null;
            }

            m_objMediaEvent = m_objFilterGraph as IMediaEvent;
            m_objMediaEventEx = m_objFilterGraph as IMediaEventEx;
            m_objMediaEventEx.SetNotifyWindow((int)this.Handle, WM_GRAPHNOTIFY, 0);
            m_objMediaPosition = m_objFilterGraph as IMediaPosition;
            m_objMediaControl = m_objFilterGraph as IMediaControl;

            //this.Text = "DirectShow - [" + openFileDialog.FileName + "]";

            m_objMediaControl.Pause();
            m_CurrentStatus = MediaStatus.Paused;

            UpdateStatusBar();
            UpdateToolBar();

        }

        private void UpdateStatusBar()
        {
            switch (m_CurrentStatus)
            {
                case MediaStatus.None: statusBarPanel1.Text = "Stopped"; break;
                case MediaStatus.Paused: statusBarPanel1.Text = "Paused "; break;
                case MediaStatus.Running: statusBarPanel1.Text = "Running"; break;
                case MediaStatus.Stopped: statusBarPanel1.Text = "Stopped"; break;
            }

            if (m_objMediaPosition != null)
            {
                int s = (int)m_objMediaPosition.Duration;
                int h = s / 3600;
                int m = (s - (h * 3600)) / 60;
                s = s - (h * 3600 + m * 60);

                statusBarPanel2.Text = String.Format("{0:D2}:{1:D2}:{2:D2}", h, m, s);

                s = (int)m_objMediaPosition.CurrentPosition;
                h = s / 3600;
                m = (s - (h * 3600)) / 60;
                s = s - (h * 3600 + m * 60);

                statusBarPanel3.Text = String.Format("{0:D2}:{1:D2}:{2:D2}", h, m, s);
            }
            else
            {
                statusBarPanel2.Text = "00:00:00";
                statusBarPanel3.Text = "00:00:00";
            }
        }

        private void UpdateToolBar()
        {
            switch (m_CurrentStatus)
            {
                case MediaStatus.None: btnPlay.Enabled = false;
                    btnPause.Enabled = false;
                    btnStop.Enabled = false;
                    break;

                case MediaStatus.Paused: btnPlay.Enabled = true;
                    btnPause.Enabled = false;
                    btnStop.Enabled = true;
                    break;

                case MediaStatus.Running: btnPlay.Enabled = false;
                    btnPause.Enabled = true;
                    btnStop.Enabled = true;
                    break;

                case MediaStatus.Stopped: btnPlay.Enabled = true;
                    btnPause.Enabled = false;
                    btnStop.Enabled = false;
                    break;
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (m_CurrentStatus == MediaStatus.Running)
            {
                UpdateStatusBar();
            }
        }

        protected override void WndProc(ref Message m)
        {
            if (m.Msg == WM_GRAPHNOTIFY)
            {
                int lEventCode;
                int lParam1, lParam2;

                while (true)
                {
                    try
                    {
                        m_objMediaEventEx.GetEvent(out lEventCode,
                            out lParam1,
                            out lParam2,
                            0);

                        m_objMediaEventEx.FreeEventParams(lEventCode, lParam1, lParam2);

                        if (lEventCode == EC_COMPLETE)
                        {
                            m_objMediaControl.Stop();
                            m_objMediaPosition.CurrentPosition = 0;
                            m_CurrentStatus = MediaStatus.Stopped;
                            UpdateStatusBar();
                            UpdateToolBar();
                        }
                    }
                    catch (Exception)
                    {
                        break;
                    }
                }
            }

            base.WndProc(ref m);
        }



        private void tsPlayListMonkey_Click(object sender, EventArgs e)
        {
            CleanUp();

            m_objFilterGraph = new FilgraphManager();

            //Path.Combine(Application.StartupPath, @"Images\sergiomedia.jpg")

            m_objFilterGraph.RenderFile(Path.Combine(Application.StartupPath, @"Images\chimp.mpg"));

            m_objBasicAudio = m_objFilterGraph as IBasicAudio;

            try
            {
                m_objVideoWindow = m_objFilterGraph as IVideoWindow;
                m_objVideoWindow.Owner = (int)panel1.Handle;
                //m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPCHILDREN;
                m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
                m_objVideoWindow.SetWindowPosition(panel1.ClientRectangle.Left,
                    panel1.ClientRectangle.Top,
                    panel1.ClientRectangle.Width,
                    panel1.ClientRectangle.Height);


            }
            catch (Exception)
            {
                m_objVideoWindow = null;
            }


            m_objMediaEvent = m_objFilterGraph as IMediaEvent;

            m_objMediaEventEx = m_objFilterGraph as IMediaEventEx;
            m_objMediaEventEx.SetNotifyWindow((int)this.Handle, WM_GRAPHNOTIFY, 0);

            m_objMediaPosition = m_objFilterGraph as IMediaPosition;


            m_objMediaControl = m_objFilterGraph as IMediaControl;

            //this.Text = "DirectShow - [" + openFileDialog.FileName + "]";

            m_objMediaControl.Run();
            m_CurrentStatus = MediaStatus.Running;

            UpdateStatusBar();
            UpdateToolBar();

        }

        private void cb1_SelectedIndexChanged(object sender, EventArgs e)
        {
            string file = Path.Combine(Application.StartupPath, @"Images\ftlogo.jpg");

            if (cb1.SelectedIndex < 0)
                return;

            if (cb1.SelectedIndex == 0)
                file = Path.Combine(Application.StartupPath, @"Images\ftlogo.jpg");

            if (cb1.SelectedIndex == 1)
                file = Path.Combine(Application.StartupPath, @"Images\ft1.wmv");

            if (cb1.SelectedIndex == 2)
                file = Path.Combine(Application.StartupPath, @"Images\ft2.wmv");


                CleanUp();

                m_objFilterGraph = new FilgraphManager();
                m_objFilterGraph.RenderFile(file);

                m_objBasicAudio = m_objFilterGraph as IBasicAudio;

                try
                {
                    m_objVideoWindow = m_objFilterGraph as IVideoWindow;
                    m_objVideoWindow.Owner = (int)panel1.Handle;
                    //m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPCHILDREN;
                    m_objVideoWindow.WindowStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
                    m_objVideoWindow.SetWindowPosition(panel1.ClientRectangle.Left,
                        panel1.ClientRectangle.Top,
                        panel1.ClientRectangle.Width,
                        panel1.ClientRectangle.Height);

                }
                catch (Exception)
                {
                    m_objVideoWindow = null;
                }

                m_objMediaEvent = m_objFilterGraph as IMediaEvent;

                m_objMediaEventEx = m_objFilterGraph as IMediaEventEx;
                m_objMediaEventEx.SetNotifyWindow((int)this.Handle, WM_GRAPHNOTIFY, 0);

                m_objMediaPosition = m_objFilterGraph as IMediaPosition;

                m_objMediaControl = m_objFilterGraph as IMediaControl;

                //this.Text = "DirectShow - [" + openFileDialog.FileName + "]";

                m_objMediaControl.Run();
                m_CurrentStatus = MediaStatus.Running;

                UpdateStatusBar();
                UpdateToolBar();
  
        }




    }
}





/*
Find the temporary directory:
    string temppath = Path.GetTempPath();
    // will generate something like C:\Documents and Settings\youraccount\Local Settings\Temp

Generate a random file name:
    string randomFile = Path.GetRandomFileName();
    // will generate something like utbnd0fm.uyp or ewghyx4x.cvd

Generate a random text file:
You can use the ChangeExtension() method in conjunction with the GetRandomFileName() method.
    string temptext = Path.ChangeExtension(Path.GetRandomFileName(), ".txt");
    // generates something like pe054llb.txt

Get a unique temp file:
Ever need to generate a uniquely named file for some temporary operation? GetTempFileName() will actually generate a 0 byte file on the file system and return you the name.
    string tempfile = Path.GetTempFileName();
    // will generate something like C:\Documents and Settings\youraccount\Local Settings\Temp\tmp62.tmp

Get file name from a full path:
    string filepath = "c:\\windows\\somefile.txt"
    file = Path.GetFileName(filepath); // returns just somefile.txt

Get the directory name from a file path:
    string filepath = "c:\\windows\\somefile.txt"
    file = Path.GetDirectoryName(filepath); // returns just c:\windows

Get the file extension:
    string filepath = "c:\\windows\\somefile.txt"
    file = Path.GetExtension(filepath); // returns just .txt

Find invalid file name characters:
The GetInvalidFileNameChars() will return an array of chars that are invalid to use in a file name. You can use this to check a user supplied file name before attempting to write a file to disk. There is a correspnding method for invalid path chars as well, GetInvalidPathChars().
    string userfile = "junk.t>t";
    char[] userchars = userfile.ToCharArray();
    char[] invalids = Path.GetInvalidFileNameChars();
    foreach (char c in userchars)
    {
        foreach (char i in invalids)
        {
            if (c == i)
                throw new Exception("File name contains invalid chars!");
        }
    }
*/


